from django.db import models
from localflavor.generic.countries.sepa import IBAN_SEPA_COUNTRIES
from localflavor.generic.models import IBANField


class Azienda(models.Model):
    SDI = "SDI"
    PEC = "PEC"
    PREFERENZA_INVIO_FEL = ((SDI, "Codice destinatario SDI"), (PEC, "Indirizzo PEC"))

    last_update = models.DateTimeField(auto_now_add=True)

    """Ragione sociale dell'azienda, così com'è iscritta al REA"""
    denominazione = models.CharField(
        blank=False, max_length=500, help_text="Il nome dell'azienda", verbose_name="Ragione Sociale"
    )

    id_fiscale_IVA_paese = models.CharField(
        blank=False, default="IT", max_length=2, verbose_name="Codice Paese ID fiscale IVA"
    )

    id_fiscale_IVA_codice = models.CharField(
        blank=True, null=True, default="99999999999", max_length=11, verbose_name="Partita IVA"
    )

    codice_fiscale = models.CharField(blank=True, null=True, default=None, max_length=16)

    codice_destinatario_SDI = models.CharField(
        null=True, default="0000000", max_length=7, verbose_name="Codice Destinatario SDI"
    )

    indirizzo_PEC = models.EmailField(blank=True, null=True)

    preferenza_invio_FEL = models.CharField(
        blank=True, null=True, max_length=3, default=None, choices=PREFERENZA_INVIO_FEL
    )

    """Il nome dell'indirizzo senza numero civico"""
    sede_indirizzo = models.CharField(max_length=500, verbose_name="Indirizzo (SEDE)", help_text="Senza numero civico")

    """Il numero civico dell'indirizzo della sede"""
    sede_n_civico = models.CharField(max_length=50, verbose_name="Numero Civico (SEDE)", blank=True, null=True)

    """Il codice di avviamento postale della sede"""
    sede_cap = models.CharField(max_length=10, verbose_name="CAP (SEDE)")

    sede_nazione = models.CharField(max_length=2, default="IT", verbose_name="Nazione (SEDE)")

    sede_provincia = models.CharField(max_length=2, default="BS", verbose_name="Provincia (SEDE)")

    sede_comune = models.CharField(max_length=255, default="Mazzano", verbose_name="Comune (SEDE)")

    telefono = models.CharField(
        blank=True, null=True, max_length=16, help_text="Seguire le norme internazionali: +390302122560"
    )

    fax = models.CharField(
        blank=True, null=True, max_length=16, help_text="Seguire le norme internazionali: +390302122560"
    )

    email = models.EmailField(blank=True, null=True)

    nome_banca = models.CharField(blank=True, null=True, default=None, max_length=255, verbose_name="Nome della Banca")

    IBAN = IBANField(include_countries=IBAN_SEPA_COUNTRIES, blank=True, null=True)

    @property
    def indirizzo_sede(self):
        return "{sede_indirizzo}, {sede_n_civico}, {sede_cap} {sede_comune} ({sede_provincia})".format(
            sede_indirizzo=self.sede_indirizzo,
            sede_n_civico=self.sede_n_civico or "SNC",
            sede_cap=self.sede_cap,
            sede_comune=self.sede_comune,
            sede_provincia=self.sede_provincia,
        )

    def __str__(self):
        denominazione = (self.denominazione[:50] + "..") if len(self.denominazione) > 75 else self.denominazione

        partita_iva = self.id_fiscale_IVA_paese + self.id_fiscale_IVA_codice

        return f"{denominazione} ({partita_iva})"

    class Meta:
        verbose_name = "azienda"
        verbose_name_plural = "aziende"
        ordering = ["denominazione"]
